#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os,atexit,signal,subprocess,time,shlex
from sys import *
import pygtk
pygtk.require("2.0")
import gtk,gobject
from gtk import gdk
try:
    from xfce4 import netk as wnck
except ImportError:
    import wnck



from conf import Conf
conf=Conf('~/.qconrc')





def handle_signal(signum,*args):
    term_instance.toggle_window()

def terminate(*args):
#    print >>stderr,'Interrupted...'
    term_instance.kill()

if '-v' in argv:
    def debug(*msg):
        msg=" ".join(map(str,msg))
        print >>stderr,msg
else:
    def debug(*msg):
        pass



class PidFile(object):
    filename = property(lambda self:os.path.expanduser('~/.qcon.pid'))
    togglesh = property(lambda self:os.path.expanduser('~/.qcon-toggle'))
    def __init__(self):
        self.pid=os.getpid()
        try: oldpid=int(open(self.filename).readline().strip())
        except: oldpid=None
        if oldpid:
            try:
                os.kill(oldpid,0)
                print >>stderr,'Already running'
                exit(1)
            except OSError:
                pass
        open(self.filename,'w').write(str(self.pid))
        open(self.togglesh,'w').write("#!/bin/sh\nexec kill -USR1 %d\n" % self.pid)
        os.chmod(self.togglesh,0755)
        atexit.register(self.unlink)
    def unlink(self):
        debug('Removing pid file')
        os.unlink(self.filename)
        open(self.togglesh,'w').write("#!/bin/sh\nqcon\n")
        os.chmod(self.togglesh,0755)

class Term(object):
    def __init__(self,cmd):
        self._cmd=cmd
        self.screen=wnck.screen_get_default()
        self.run()
        if not globals().has_key('term_instance'):
            global term_instance
            term_instance=self
            signal.signal(signal.SIGUSR1,handle_signal)
            atexit.register(Term.kill,self)
    def run(self):
        self.proc=subprocess.Popen(tuple(shlex.split(self._cmd)))
        self.window=None
        self._watch_id=gobject.child_watch_add(self.proc.pid,self.on_child_exit)
        gobject.idle_add(self.find_window)
        self._find_timeout_id=gobject.timeout_add(conf.FIND_WINDOW_TIMEOUT*1000,self.find_timeout)
    def on_child_exit(self,cpid,errlev):
        debug('Terminal process with pid %d died with exit status %d'%(cpid,errlev))
        self.proc.returncode=errlev
        if conf.RESPAWN:
            self.run()
        else:
            exit(errlev)
    def find_window(self):
        for w in self.screen.get_windows():
            if w.get_pid() == self.proc.pid:
                self.window=TermWindow(w,self)
                gobject.source_remove(self._find_timeout_id)
                return False
        return True
    def find_timeout(self):
        if not self.window:
            print >>stderr,"No terminal window for %d seconds, terminating" % conf.FIND_WINDOW_TIMEOUT
            exit(3)
    def toggle_window(self):
        self.window and self.window.toggle()
    def kill(self):
        gobject.source_remove(self._watch_id)
        if self.proc.poll() is None:
            debug('Killing terminal process')
            try:
                os.kill(self.proc.pid,signal.SIGINT)
            except OSError:
                pass
    
class TermWindow(object):
    def __init__(self,window,termproc):
        self.term=termproc
        self.window=window
        self.wrapped=gdk.window_foreign_new(window.get_xid())
        self.setup()
        self._active=False
        self.term.screen.connect('active_window_changed',self.hide_inactive)
    def setup(self):
        self.wrapped.set_skip_pager_hint(True)
        self.wrapped.set_keep_above(True)
        self.wrapped.set_decorations(gdk.DECOR_BORDER)
        #self.wrapped.set_functions(gdk.FUNC_RESIZE | gdk.FUNC_MINIMIZE)
        self.wrapped.set_functions(0 | gdk.FUNC_MINIMIZE)
        self.wrapped.move(*(conf._geom(self.term.screen)[0]))
        self.wrapped.resize(*(conf._geom(self.term.screen)[1]))
        try:
            self.wrapped.set_icon_list([gtk.gdk.pixbuf_new_from_file(
                os.path.join(os.path.dirname(__file__),'..','share/pixmaps/qcon.png') ),])
        except:
            print >>stderr,"Can't set icon"
    #def to_corner(self,descr):
    #    descr=descr.lower()
    #    w,h=self.wrapped.get_size()
    #    if 'left' in descr: x=0
    #    elif 'right' in descr: x=self.term.screen.get_width()-w
    #    else: x=self.term.screen.get_width()/2-(w/2)
    #    if 'top' in descr: y=0
    #    elif 'bottom' in descr: y=self.term.screen.get_height()-h
    #    else: y=self.term.screen.get_height()/2-(h/2)
    #    print x,y,'top' in descr
    #    self.wrapped.move(x,y)
    def activate(self):
        self.wrapped.set_skip_taskbar_hint(False)
        self.wrapped.deiconify()
        self.wrapped.focus()
        self._active=True
        #self.window.set_skip_tasklist(False);self.window.unminimize();self.window.activate()
    def hide(self):
        self.wrapped.iconify()
        self.wrapped.set_skip_taskbar_hint(True)
        self._active=False
        #self.window.minimize();self.window.set_skip_tasklist(True)
    def toggle(self):
        if self.window.is_active():
            debug('toggle:hide');self.hide()
        else:
            debug('toggle:activate');self.activate()
    def hide_inactive(self,scrobj=None):
        active_pid=self.term.screen.get_active_window()
        if not self._active and active_pid==self.window:
            self.activate()
        elif self._active:
            active_pid=active_pid and active_pid.get_pid() or 0
            if active_pid != self.term.proc.pid:
                debug('hide_inactive')
                self.hide()
        return False





def main():
    global conf
    if not conf._exists:
        import settings
        settings.main()
        conf=Conf('~/.qconrc')
    pidfile=PidFile()
    #signal.signal(signal.SIGINT,terminate)
    term=Term(conf.COMMAND)


    loop=gobject.MainLoop()
    loop.run()


if __name__=='__main__':
    main()

